/*	Test program for custom trig functions.
 * March 6, 2022
 * Dirk Mittler
 */

#include "functions.h"

#include <iostream>
#include <iomanip>
#include <string>
#include <algorithm>

using std::cout;
using std::cin;
using std::endl;
using std::setprecision;
using std::string;

#define DIGITS2 std::numeric_limits<double>::digits
const double FP_MIN = 1.0 / (1UL << (DIGITS2 / 5));


int main(int argc, char* argv[])
{
    //  The results array will have 6 fields per entry,
    //	Index,
    //  Argument, Real Sine, Real Arc-Sine, Test Sine, Test Arc-Sine...
    //  There will be 20 entries.

    const int columns = 6;
    const double excursion = M_PI * 0.5;
    double* results = new double[120];
    int index = 0;
    double arg = 0.0;
    const double v1 = M_PI;
    const double v2 = M_PI * 101;
    const double v3 = sqrt(0.5);
    string cpu = "(Unknown)";
    string marz = "(Other)";
    const int decimals = std::numeric_limits<double>::digits10;
    string in_buff = "33";

#ifdef _M_AMD64
    cpu = "amd64";
#endif

#ifdef __x86_64__
    cpu = "amd64";
#endif

#ifdef _M_ARM
    cpu = "arm";
#endif

#ifdef __arm__
    cpu = "arm";
#endif

#ifdef _M_IX86
    cpu = "x86";
#endif

#ifdef __i386__
    cpu = "x86";
#endif

#ifdef _M_PPC
    cpu = "PowerPC";
#endif

#ifdef __powerpc__
    cpu = "PowerPC";
#endif

#ifdef __m68k__
    cpu = "Motorola68k";
#endif

#ifdef __mc68000__
    cpu = "Motorola68000";
#endif

#ifdef __GNUC__
    marz = "GNU";
#endif

#ifdef _MSC_VER
    marz = "VStudio";
#endif

#ifdef __MINGW32__
    marz = "MinGW32";
#endif

#ifdef __MINGW64__
    marz = "MinGW-w64_64_Bit";
#endif

#ifdef __DMC__
    marz = "DigitalMarz";
#endif

    for (index = 0; index < 20 ; index++) {
        arg = (-1.0 + (index * 0.1)) * excursion;
        results[index * columns] = index - 10;
        results[index * columns + 1] = arg;
        results[index * columns + 2] = sin(arg);
        results[index * columns + 3] = abs(arg) <= 1 ? asin(arg) : 0.0;
        results[index * columns + 4] = TSin(arg);
        results[index * columns + 5] = abs(arg) <= 1 ? TAsin(arg) : 0.0;
    }

    cout << "CPU architecture at compile-time: " << cpu << endl;
    cout << "Compiler " << marz << " set to 'double'" << endl;
    cout << "setprecsion(" << decimals << ")" << endl << endl;

    cout << "* M_PI / 20" << endl;
    cout << "arg" << endl << "sin" << endl << "asin" << endl;
    cout << "TSin" << endl << "TAsin" << endl << endl;

    cout << setprecision(decimals);

    for (index = 0; index < 20; index++) {
        cout << results[index * columns] << endl;
        cout << results[index * columns + 1] << endl;
        cout << results[index * columns + 2] << endl;
        cout << results[index * columns + 3] << endl;
        cout << results[index * columns + 4] << endl;
        cout << results[index * columns + 5] << endl;
        cout << endl;
    }

    cout << "The test version of sin(M_PI) is: " << TSin(v1) << endl;
    cout << "The test version of sin(M_PI*101) is: " << TSin(v2) << endl;
    cout << "The test version of asin(sqrt(2)/2) is within: ";

    cout << TAsin(v3) - asin(v3) << endl << endl;

    cout << "Testing for endless loop at x=FP_MIN... " << endl;
    cout << TSin(FP_MIN) << " (Passed test.)" << endl;
    cout << "Testing for endless loop at x=FP_MIN*2... " << endl;
    cout << TSin(FP_MIN * 2) << " (Passed test.)" << endl << endl;

    cout << "Press Enter key to end program." << endl << endl;

    delete[] results;
    results = NULL;

    getline(cin, in_buff);

    return 0;
}
